@@ -180,6 +180,9 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
                         type, adjust, simple, combine = FALSE, ratios = TRUE, 
                         parens, ...) 
 {
+    if (!missing(type))
+        options = as.list(c(options, predict.type = type))
+    
     if(!missing(simple))
         return(.simcon(object, method = method, interaction = interaction,
                       offset = offset, scale = scale, name = name, options = options,
@@ -192,7 +195,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     
     nesting = object@model.info$nesting
     if (!is.null(nesting) || !is.null(object@misc$display))
-        return (.nested_contrast(rgobj = object, method = method, by = by, adjust = adjust, parens = parens, ...))
+        return (.nested_contrast(rgobj = object, method = method, interaction = interaction, by = by, adjust = adjust, type = type, offset = offset, ...))
     
     orig.grid = object@grid[, , drop = FALSE]
     orig.grid[[".wgt."]] = orig.grid[[".offset."]] = NULL
@@ -310,8 +313,10 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     cmat = method(levs, ...)
     if (!is.data.frame(cmat))
         stop("Contrast function must provide a data.frame")
-    else if(ncol(cmat) == 0)
+    else if(ncol(cmat) == 0) {
         cmat = data.frame(`(nothing)` = rep(NA, nrow(args)), check.names = FALSE)
+        adjust = "none"
+    }
     # warning("No contrasts were generated! Perhaps only one emmean is involved.\n",
     #      "  This can happen, for example, when your predictors are not factors.")
     else if (nrow(cmat) != nrow(args))
@@ -325,8 +330,16 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
                  " does not conform with ", nrow(tcmat), " contrasts", call. = FALSE)
     }
     
+    # do some bookkeeping whereby we get NAs only when NA rows get nonzero weight
+    NAflag = linfct[, 1] * 0
+    NAflag[is.na(linfct[, 1])] = 1
+    linfct[is.na(linfct)] = 0 # now we don't have any NAs but we know where to put them back later
+    .NArows = function(mat, flag) # utility for flagging bad rows (any nonzero coef applied to an NA)
+        apply(mat, 1, function(x, f) any(x * f != 0), flag)
+    
     if (is.null(by)) {
         linfct = tcmat %*% linfct
+        linfct[.NArows(tcmat, NAflag), ] = NA # put back the NAs where they belong
         grid = data.frame(.contrast.=names(cmat))
         if (hasName(object@grid, ".offset."))
             grid[[".offset."]] = t(cmat) %*% object@grid[[".offset."]]
@@ -338,6 +351,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     else {
         tcmat = kronecker(.diag(rep(1,length(by.rows))), tcmat)
         linfct = tcmat %*% linfct[unlist(by.rows), , drop = FALSE]
+        linfct[.NArows(tcmat, NAflag[unlist(by.rows)]), ] = NA
         tmp = expand.grid(con = names(cmat), by = seq_len(length(by.rows)), stringsAsFactors = FALSE)###unique(by.id))
         # check if levs have different orderings in subsequent by groups
         for (i in 1 + seq_along(by.rows[-1])) { 
@@ -353,7 +367,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
         xlevs = list()
         for (v in by)
             xlevs[[v]] = rep(bylevs[row.1st, v], each=n.each)
-        grid = cbind(grid, as.data.frame(xlevs))
+        grid = cbind(grid, data.frame(xlevs, check.names = FALSE))
         if (hasName(object@grid, ".offset."))
             grid[[".offset."]] = tcmat %*% object@grid[unlist(by.rows), ".offset."]
     }
@@ -379,7 +393,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     }
     misc$methDesc = attr(cmat, "desc")
     misc$famSize = ifelse(is.null(fs <- attr(cmat, "famSize")), length(by.rows[[1]]), fs)
-    misc$pri.vars = setdiff(names(grid), c(".offset.",".wgt."))
+    misc$pri.vars = setdiff(names(grid), c(by, ".offset.",".wgt."))
     if (missing(adjust)) adjust = attr(cmat, "adjust")
     if (is.null(adjust)) adjust = "none"
     if (!is.null(attr(cmat, "offset")))
@@ -395,6 +409,8 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     misc$adjust = adjust
     misc$infer = c(FALSE, TRUE)
     misc$by.vars = by
+    if(!is.na(misc$estType) && misc$estType == "pairs") # internal flag to keep track of original by vars for paired comps
+        misc$.pairby = paste(c("", by), collapse = ",")
     # save contrast coefs
     by.cols = seq_len(ncol(tcmat))
     if(!is.null(by.rows))
@@ -411,7 +427,8 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     # zap the transformation info except in special cases
     if (!is.null(misc$tran)) {
         misc$orig.tran = .fmt.tran(misc)
-        if (ratios && true.con && misc$tran %in% c("log", "genlog", "logit")) {
+        if (ratios && true.con && misc$tran %in% c("log", "log2", "log10", ### REMOVED "genlog", 
+                                                   "logit", "log.o.r.")) {
             misc$log.contrast = TRUE      # remember how we got here; used by summary
             misc$orig.inv.lbl = misc$inv.lbl
             if (misc$tran == "logit") {
@@ -419,9 +436,12 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
                 misc$tran = "log.o.r."
                 misc$tran.mult = misc$tran.offset = NULL
             }
+            else if (misc$tran == "log.o.r.") {
+                # leave everything as-is. Once an odds ratio, always an odds ratio
+            }
             else {
                 misc$inv.lbl = "ratio"
-                misc$tran = "log"
+                ### (stays at log, log2, log10)   misc$tran = "log"
                 misc$tran.mult = misc$tran.offset = NULL
             }
         }
@@ -449,9 +469,6 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     result@misc = misc
     result@roles$predictors = setdiff(names(result@levels), result@roles$multresp)
     
-    if (!missing(type))
-        options = as.list(c(options, predict.type = type))
-    
     .update.options(result, options, ...)
 }
 
@@ -473,7 +490,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
         if(combine)
             result = do.call(rbind.emmGrid, result)
     }
-    else if ((simple == "each") && !("each" %in% names(object@levels))) {
+    else if ((length(simple) == 1) && (simple == "each") && !("each" %in% names(object@levels))) {
         result = .simcon(object, ..., 
             simple = as.list(names(object@levels)), combine = combine)
     }
