@@ -76,6 +76,12 @@ str.emmGrid <- function(object, ...) {
             showlevs(levs[[nm]])
         cat("\n")
     }
+    if (length(nuis <- object@roles$nuisance) > 0) {
+        cat("Nuisance factors that have been collapsed by averaging:\n    ")
+        tmp = paste0(names(nuis), "(", sapply(nuis, length), ")")
+        cat(paste(tmp, collapse = ", "))
+        cat("\n")
+    }
     if(!is.null(object@model.info$nesting)) {
         cat("Nesting structure:  ")
         cat(.fmt.nest(object@model.info$nesting))
@@ -90,11 +96,18 @@ str.emmGrid <- function(object, ...) {
 
 
 #' @rdname emmGrid-methods
+#' @param export Logical value. If \code{FALSE}, the object is printed. 
+#'   If \code{TRUE}, a list is invisibly returned, which contains character
+#'   elements named \code{summary} and \code{annotations} that may be saved 
+#'   or displayed as the user sees fit. \code{summary} is a character matrix 
+#'   (or list of such matrices, if a \code{by} variable is in effect).
+#'   \code{annotations} is a character vector of the annotations that would 
+#'   have been printed below the summary or summaries.
 #' @method print emmGrid
 #' @param x An \code{emmGrid} object
 #' @export
-print.emmGrid = function(x,...)
-    print(summary.emmGrid(x, ...))
+print.emmGrid = function(x, ..., export = FALSE)
+    print(summary.emmGrid(x, ...), export = export)
 
 
 # vcov method
@@ -147,6 +160,7 @@ vcov.emmGrid = function(object, ...) {
 #' @return an updated \code{emmGrid} object.
 #' 
 #' @method update emmGrid
+#' @order 1
 #' @export
 #' 
 #' @section Details:
@@ -489,6 +503,14 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   main effects but included them in interactions. This does not change the
 #'   model fit, but it produces a different parameterization that is picked
 #'   up when the reference grid is constructed. Defaults to \code{TRUE}.}
+#' \item{\code{rg.limit}}{An integer value setting a limit on the number of rows
+#'   in a newly constructed reference grid. This is checked based on the number of
+#'   levels of the factors involved; but it excludes the levels of any multivariate
+#'   responses because those are not yet known. The reference grid consists of all
+#'   possible combinations of the predictors, and this can become huge if there are
+#'   several factors. An error is thrown if this limit is exceeded. One can use the 
+#'   \code{nuisance} argument of \code{\link{ref_grid}} to collapse on nuisance
+#'   factors, thus making the grid smaller. Defaults to 10,000.}
 #' \item{\code{simplify.names}}{A logical value controlling whether to
 #'   simplify (when possible) names in the model formula that refer to datasets --
 #'   for example, should we simplify a predictor name like \dQuote{\code{data$trt}}
@@ -515,7 +537,8 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #' rank-deficient cases. If absent, the value in \code{emm_defaults$estble.tol)}
 #' is used.}
 #' \item{\code{save.ref_grid}}{Logical value of \code{TRUE} if you wish the 
-#' latest reference grid created to be saved in \code{.Last.ref_grid}}
+#' latest reference grid created to be saved in \code{.Last.ref_grid}.
+#' The default is \code{FALSE}.}
 #' \item{Options for \code{lme4::lmerMod} models}{Options \code{lmer.df},
 #' \code{disable.pbkrtest}, \code{pbkrtest.limit}, \code{disable.lmerTest},
 #' and \code{lmerTest.limit}
@@ -628,7 +651,7 @@ emm_defaults = list (
     ref_grid = list(is.new.rg = TRUE, infer = c(FALSE, FALSE)),
     emmeans = list(infer = c(TRUE, FALSE)),
     contrast = list(infer = c(FALSE, TRUE)),
-    save.ref_grid = TRUE,     # save new ref_grid in .Last.ref_grid
+    save.ref_grid = FALSE,    # save new ref_grid in .Last.ref_grid
     cov.keep = "2",           # default for cov.keep arg in ref_grid
     sep = " ",                # separator for combining factor levels
     parens = c("-|\\+|\\/|\\*", "(", ")"), # patterns for what/how to parenthesize in contrast
@@ -641,6 +664,7 @@ emm_defaults = list (
     back.bias.adj = FALSE,    # Try to bias-adjust back-transformations?
     opt.digits = TRUE,        # optimize displayed digits?
     enable.submodel = TRUE,   # enable saving extra info for submodel
+    rg.limit = 10000,         # limit on number of rows in a reference grid
     lmer.df = "kenward-roger",  # Use Kenward-Roger for df
     disable.pbkrtest = FALSE, # whether to bypass pbkrtest routines for lmerMod
     pbkrtest.limit = 3000,    # limit on N for enabling K-R
@@ -651,16 +675,17 @@ emm_defaults = list (
 
 # override levels<- method
 #' @rdname update.emmGrid
+#' @order 5
 #' @export
 #' @param x an \code{emmGrid} object
 #' @param value \code{list} or replacement levels. See the documentation for
-#' \code{\link{update.emmGrid}} with the \code{levels} argument, 
+#' \code{update.emmGrid} with the \code{levels} argument, 
 #' as well as the section below on \dQuote{Replaciong levels}
 #'   
 #' @return \code{levels<-} replaces the levels of the object in-place.
-#'   See the section on  for details.
+#'   See the section on replacing levels for details.
 #' @section Replacing levels:
-#' The \code{levels<-} method uses \code{\link{update.emmGrid}} to replace the
+#' The \code{levels<-} method uses \code{update.emmGrid} to replace the
 #' levels of one or more factors. This method allows selectively replacing
 #' the levels of just one factor (via subsetting operators), whereas 
 #' \code{update(x, levels = list(...))} requires a list of \emph{all} factors
@@ -950,9 +975,14 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     
     est = .est.se.df(object, do.se = TRUE) ###FALSE)
     estble = !(is.na(est[[1]]))
-    object@V = vcov(object)[estble, estble, drop=FALSE]
+    object@V = vcov(object)[estble, estble, drop = FALSE]
     object@bhat = est[[1]]
     object@linfct = diag(1, length(estble))
+    if (!is.null(disp <- object@misc$display)) {  # fix up for the bookkeeping in nested models
+        object@V = object@V[disp, disp, drop = FALSE]
+        object@linfct = matrix(0, nrow = length(disp), ncol = length(estble))
+        object@linfct[disp, ] = diag(1, length(estble))
+    }
     if(all(estble))
         object@nbasis = estimability::all.estble
     else
