@@ -75,8 +75,8 @@ glht.emmlf <- function(model, linfct, ...) {
 # Note: model is redundant, really, so can be omitted
 # See related roxygen stuff just before glht.emmlf
 glht.emmGrid <- function(model, linfct, by, ...) {
-    if (!requireNamespace("multcomp"))
-        stop(sQuote("glht")," requires ", dQuote("multcomp"), " to be installed")
+    .requireNS("multcomp", sQuote("glht")," requires ", dQuote("multcomp"), 
+               " to be installed", call. = FALSE)
     object = linfct # so I don't get confused
     if (missing(model)) 
         model = .cls.list("emmwrap", object = object)
@@ -102,19 +102,19 @@ glht.emmGrid <- function(model, linfct, by, ...) {
     
     if (is.null(by)) {
         args$linfct = lf
-        return(do.call("glht", args))
+        return(do.call(multcomp::glht, args))
     }
     
     # (else...)
     by.rows = .find.by.rows(object@grid, by)
     result = lapply(by.rows, function(r) {
         args$linfct = lf[r, , drop=FALSE]
-        do.call("glht", args)
+        do.call(multcomp::glht, args)
     })
     bylevs = lapply(by, function(byv) unique(object@grid[[byv]]))
     names(bylevs) = by
     bygrid = do.call("expand.grid", bylevs)
-    levlbls = lapply(by, function(byv) paste(byv, "=", bygrid[[byv]]))
+    levlbls = unname(lapply(by, function(byv) paste(byv, "=", bygrid[[byv]])))
     levlbls$sep = ", "
     names(result) = do.call("paste", levlbls)
     class(result) = c("glht_list", "list")
@@ -142,22 +142,16 @@ glht.emmGrid <- function(model, linfct, by, ...) {
 #'   The user may override this via the \code{df} argument.
 #' 
 #' @examples
-#' if(require(multcomp)) { # --- multcomp must be installed
-#' 
+#' if(require(multcomp, quietly = TRUE)) withAutoprint({ # --- multcomp must be installed
 #' warp.lm <- lm(breaks ~ wool*tension, data = warpbreaks)
-#' 
 #' # Using 'emm'
 #' summary(glht(warp.lm, emm(pairwise ~ tension | wool)))
-#' 
 #' # Same, but using an existing 'emmeans' result
 #' warp.emm <- emmeans(warp.lm, ~ tension | wool)
 #' summary(as.glht(pairs(warp.emm)))
-#' 
 #' # Same contrasts, but treat as one family
 #' summary(as.glht(pairs(warp.emm), by = NULL))
-#' 
-#' } # --- was tested only if multcomp is installed
-#' @rdname glht-support
+#' }, spaced = TRUE)
 #' @export
 as.glht <- function(object, ...) {
     UseMethod("as.glht")
