@@ -25,11 +25,14 @@
 
 #--------------------------------------------------------------
 ### lm objects (and also aov, rlm, others that inherit) -- but NOT aovList
+### Recent additional arhument 'frame' should point to where the model frame 
+###   might be available, or NULL otherwise
 #' @method recover_data lm
 #' @export
-recover_data.lm = function(object, ...) {
+recover_data.lm = function(object, frame = object$model, ...) {
         fcall = object$call
-    recover_data(fcall, delete.response(terms(object)), object$na.action, ...)
+    recover_data(fcall, delete.response(terms(object)), object$na.action, 
+                 frame = frame, ...)
 }
 
 #' @export
@@ -38,7 +41,9 @@ emm_basis.lm = function(object, trms, xlev, grid, ...) {
     bhat = object$coefficients
     nm = if(is.null(names(bhat))) row.names(bhat) else names(bhat)
     m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
-    X = model.matrix(trms, m, contrasts.arg = object$contrasts)[, nm, drop = FALSE]
+    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
+    assign = attr(X, "assign")
+    X = X[, nm, drop = FALSE]
     bhat = as.numeric(bhat) 
     # stretches it out if multivariate - see mlm method
     V = .my.vcov(object, ...)
@@ -50,14 +55,16 @@ emm_basis.lm = function(object, trms, xlev, grid, ...) {
     misc = list()
     if (inherits(object, "glm")) {
         misc = .std.link.labels(object$family, misc)
-        dffun = function(k, dfargs) Inf
-        dfargs = list()
+        dffun = function(k, dfargs) dfargs$df
+        dfargs = list(df = ifelse(object$family$family %in% c("gaussian", "Gamma"), 
+                                  object$df.residual, Inf))
     }
     else {
         dfargs = list(df = object$df.residual)
         dffun = function(k, dfargs) dfargs$df
     }
-    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
+    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc,
+         model.matrix = .cmpMM(object$qr, assign = assign))
 }
 
 
@@ -84,7 +91,8 @@ emm_basis.mlm = function(object, trms, xlev, grid, ...) {
 # manova objects
 recover_data.manova = function(object, ...) {
     fcall = match.call(aov, object$call)   # need to borrow arg matching from aov()
-    recover_data(fcall, delete.response(terms(object)), object$na.action, ...)
+    recover_data(fcall, delete.response(terms(object)), object$na.action, 
+                 frame = object$model, ...)
 }
 
 
@@ -98,7 +106,7 @@ recover_data.merMod = function(object, ...) {
         return("Can't handle a nonlinear mixed model")
     fcall = object@call
     recover_data(fcall, delete.response(terms(object)), 
-                 attr(object@frame, "na.action"), ...)
+                 attr(object@frame, "na.action"), frame = object@frame, ...)
 }
 
 #' @export
@@ -220,7 +228,11 @@ emm_basis.merMod = function(object, trms, xlev, grid, vcov.,
     else
         nbasis=estimability::all.estble
     
-    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
+    mm = .cmpMM(object@pp$X, object@pp$Xwts^2,
+                attr(object@pp$X, "assign"))
+    
+    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc,
+         model.matrix = mm)
 }
 
 
@@ -241,18 +253,23 @@ recover_data.lme = function(object, data, ...) {
         else
             fcall$weights = nlme::varWeights(object$modelStruct)
     }
-    recover_data(fcall, delete.response(object$terms), object$na.action, data = data, ...)
+    dat = recover_data(fcall, delete.response(object$terms), object$na.action, data = data, ...)
+    attr(dat, "pass.it.on") = TRUE
+    dat
 }
 
 #' @export
 emm_basis.lme = function(object, trms, xlev, grid, 
-        mode = c("containment", "satterthwaite", "boot-satterthwaite", "auto"), 
-        sigmaAdjust = TRUE, options, ...) {
+        mode = c("containment", "satterthwaite", "appx-satterthwaite", "auto", "boot-satterthwaite", "asymptotic"), 
+        sigmaAdjust = TRUE, options, extra.iter = 0, ...) {
     mode = match.arg(mode)
+    if (mode == "boot-satterthwaite") mode = "appx-satterthwaite"  # backward compatibility
+    if (mode == "asymptotic")
+        options$df = Inf
     if (!is.null(options$df)) # if we're gonna override the df anyway, keep it simple!
         mode = "fixed"
     if (mode == "auto")
-        mode = ifelse(is.null(object$apVar), "containment", "boot-satterthwaite")
+        mode = ifelse(is.null(object$apVar), "containment", "appx-satterthwaite")
     if (is.null(object$apVar))
         mode = "containment"
     contrasts = object$contrasts
@@ -272,8 +289,9 @@ emm_basis.lme = function(object, trms, xlev, grid,
         dfargs = list(df = options$df)
         dffun = function(k, dfargs) dfargs$df
     }
-    else if (mode %in% c("satterthwaite", "boot-satterthwaite")) {
-        G = try(gradV.kludge(object), silent = TRUE)
+    else if (mode %in% c("satterthwaite", "appx-satterthwaite")) {
+        mode = "appx-satterthwaite"
+        G = try(gradV.kludge(object, extra.iter = extra.iter), silent = TRUE)
         ###! not yet, doesn't work G = try(lme_grad(object, object$call, object$data, V))
         if (inherits(G, "try-error"))
             stop("Unable to estimate Satterthwaite parameters")
@@ -297,8 +315,15 @@ emm_basis.lme = function(object, trms, xlev, grid,
         dfargs = list(dfx = dfx)
     }
     attr(dffun, "mesg") = mode
+    
+    # submodel support (not great -- omits any weights)
+    m = model.frame(trms, attr(object, "data"), na.action = na.pass, xlev = xlev)
+    mm = model.matrix(trms, m, contrasts.arg = contrasts)
+    mm = .cmpMM(mm, assign = attr(mm, "assign"))
+    
     list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
-         dffun = dffun, dfargs = dfargs, misc = misc)
+         dffun = dffun, dfargs = dfargs, misc = misc,
+         model.matrix = mm)
 }
 
 # Here is a total hack, but it works pretty well
@@ -306,16 +331,26 @@ emm_basis.lme = function(object, trms, xlev, grid,
 # model with a few random perturbations of y, then
 # regressing the changes in V against the changes in the 
 # covariance parameters
-gradV.kludge = function(object, Vname = "varFix", call = object$call$fixed, data = object$data) {
+gradV.kludge = function(object, Vname = "varFix", call = object$call$fixed, 
+                        data = object$data, extra.iter = 0) {
+    # check consistency of contrasts
+    #### This code doesn't work with coerced factors. Hardly seems messing with, so I commented it out
+    # cnm = names(object$contrasts)
+    # cdiff = sapply(cnm, function(.) max(abs(contrasts(data[[.]]) - object$contrasts[[.]])))
+    # if (max(cdiff) > 1e-6) {
+    #     message("Contrasts don't match those used when the model was fitted. Fix this and re-run")
+    #     stop()
+    # }
+    
     A = object$apVar
     theta = attr(A, "Pars")
     V = object[[Vname]]
     sig = .01 * object$sigma
     #data = object$data
-    yname = all.vars(call)[1]
+    yname = all.vars(eval(call))[1]
     y = data[[yname]]
     n = length(y)
-    dat = t(replicate(2 + length(theta), {
+    dat = t(replicate(2 + extra.iter + length(theta), {
         data[[yname]] = y + sig * rnorm(n)
         mod = update(object, data = data)
         c(attr(mod$apVar, "Pars") - theta, as.numeric(mod[[Vname]] - V))
@@ -354,7 +389,7 @@ gradV.kludge = function(object, Vname = "varFix", call = object$call$fixed, data
 
 #--------------------------------------------------------------
 
-### new way to get gradients for gls models
+### new way to get jacobians for gls models
 gls_grad = function(object, call, data, V) {
     obj = object$modelStruct
     conLin = object
@@ -366,55 +401,68 @@ gls_grad = function(object, call, data, V) {
     func = function(x) {
         obj = nlme::`coef<-`(obj, value = x)
         tmp = nlme::glsEstimate(obj, conLin)
-        crossprod(tmp$sigma * tmp$varBeta)
+        .get.lt(crossprod(tmp$sigma * tmp$varBeta))  # lower triangular form
     }
     res = numDeriv::jacobian(func, coef(obj))
-    G = lapply(seq_len(ncol(res)), function(j) matrix(res[, j], ncol = ncol(V)))
+    G = lapply(seq_len(ncol(res)), function(j) .lt2mat(res[, j]))
     G[[1 + length(G)]] = 2 * V  # gradient wrt log sigma
     G
 }
 
 ### gls objects (nlme package)
-recover_data.gls = function(object, ...) {
+recover_data.gls = function(object, data, ...) {
     fcall = object$call
-    if (!is.null(fcall$weights))
-        fcall$weights = nlme::varWeights(object$modelStruct)
+    if (!is.null(wts <- fcall$weights)) {
+        wts = nlme::varWeights(object$modelStruct)
+        fcall$weights = NULL
+    }
     trms = delete.response(terms(nlme::getCovariateFormula(object)))
-    recover_data(fcall, trms, object$na.action, ...)
+    result = recover_data.call(fcall, trms, object$na.action, data = data, ...)
+    if (!is.null(wts))
+        result[["(weights)"]] = wts
+    if (!missing(data))
+        attr(result, "misc") = list(data = data)
+    attr(result, "pass.it.on") = TRUE
+    result
 }
 
 emm_basis.gls = function(object, trms, xlev, grid, 
-                         mode = c("auto", "df.error", "satterthwaite", "boot-satterthwaite"), 
-                         options, ...) {
+                         mode = c("auto", "df.error", "satterthwaite", "appx-satterthwaite", "boot-satterthwaite", "asymptotic"), 
+                         extra.iter = 0, options, misc, ...) {
     contrasts = object$contrasts
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts)
     bhat = coef(object)
     V = .my.vcov(object, ...)
     nbasis = estimability::all.estble
-    misc = list()
     mode = match.arg(mode)
+    if (mode == "boot-satterthwaite") mode = "appx-satterthwaite"  # backward compatibility
     if (!is.null(options$df)) # if we're gonna override the df anyway, keep it simple!
         mode = "df.error"
     if (mode == "auto")
         mode = ifelse(is.null(object$apVar), "df.error", "satterthwaite")
     if (!is.matrix(object$apVar))
         mode = "df.error"
-    if (mode %in% c("satterthwaite", "boot-satterthwaite")) {
+    if (mode %in% c("satterthwaite", "appx-satterthwaite")) {
+        data = if(is.null(misc$data))
+            eval(object$call$data, parent.frame(2))
+        else
+            misc$data
+        misc = list()
         chk = attr(object$apVar, "Pars")
         if(max(abs(coef(object$modelStruct) - chk[-length(chk)])) > .001) {
-            message("Analytical Satterthwaite method not available; using boot-satterthwaite")
-            mode = "boot-satterthwaite"
+            message("Analytical Satterthwaite method not available; using appx-satterthwaite")
+            mode = "appx-satterthwaite"
         }
-        if (mode == "boot-satterthwaite") {
-            G = try(gradV.kludge(object, "varBeta", call = object$call,
-                                 data = eval(object$call$data)),
+        if (mode == "appx-satterthwaite") {
+            G = try(gradV.kludge(object, "varBeta", call = object$call$model,
+                                 data = data, extra.iter = extra.iter),
                     silent = TRUE)
         }
         else
-            G = try(gls_grad(object, object$call, eval(object$call$data), V))
+            G = try(gls_grad(object, object$call, data, V))
         if (inherits(G, "try-error")) {
-            sugg = ifelse(mode == "satterthwaite", "boot-satterthwaite", "df.error")
+            sugg = ifelse(mode == "satterthwaite", "appx-satterthwaite", "df.error")
             stop("Can't estimate Satterthwaite parameters.\n",
                  "  Try adding the argument 'mode = \"", sugg, "\"'", call. = FALSE)
         }
@@ -426,12 +474,20 @@ emm_basis.gls = function(object, trms, xlev, grid,
             2 * est^2 / varest
         }
     }
-    else if (mode == "df.error") {  ### mode == "df.error"
-        dfargs = list(df = object$dims$N - object$dims$p - length(attr(object$apVar, "Pars")))
+    else if (mode %in%  c("df.error", "asymptotic")) { 
+        df = ifelse(mode == "asymptotic", Inf, object$dims$N - object$dims$p - length(attr(object$apVar, "Pars")))
+        dfargs = list(df = df)
         dffun = function(k, dfargs) dfargs$df
     }
     attr(dffun, "mesg") = mode
-    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
+    
+    # submodel support (not great because I don't know how to retrieve the weights)
+    m = model.frame(trms, attr(object, "data"), na.action = na.pass, xlev = xlev)
+    mm = model.matrix(trms, m, contrasts.arg = contrasts)
+    mm = .cmpMM(mm, assign = attr(mm, "assign"))
+    
+    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc,
+         model.matrix = mm)
 }
 
 
@@ -488,9 +544,9 @@ emm_basis.polr = function(object, trms, xlev, grid,
 recover_data.survreg = function(object, ...) {
     fcall = object$call
     trms = delete.response(terms(object))
-    # I'm gonna delete any terms involving strata(), cluster(), or frailty()
+    # I'm gonna delete any terms involving cluster(), or frailty() -- keep strata()
     mod.elts = dimnames(attr(trms, "factor"))[[2]]
-    tmp = grep("strata\\(|cluster\\(|frailty\\(", mod.elts)
+    tmp = grep("cluster\\(|frailty\\(", mod.elts)
     if (length(tmp))
         trms = trms[-tmp]
     recover_data(fcall, trms, object$na.action, ...)
@@ -502,13 +558,17 @@ recover_data.survreg = function(object, ...) {
 emm_basis.survreg = function(object, trms, xlev, grid, ...) {
     # Much of this code is adapted from predict.survreg
     bhat = object$coefficients
-    k = length(bhat)
+    k = length(bhat) - sum(is.na(bhat))
     V = .my.vcov(object, ...)[seq_len(k), seq_len(k), drop=FALSE]
     # ??? not used... is.fixeds = (k == ncol(object$var))
+    ### zap-out factors in xlev not needed by model.frame
+    xlev[setdiff(names(xlev), rownames(attr(trms, "factors")))] = NULL
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)    
     # X = model.matrix(object, m) # This is what predict.survreg does
     # But I have manipulated trms, so need to make sure things are consistent
     X = model.matrix(trms, m, contrasts.arg = object$contrasts)
+    usecols = intersect(colnames(X), names(bhat))
+    bhat = bhat[usecols]  # in case ref_grid code excluded some levels...
     nbasis = estimability::nonest.basis(model.matrix(object))
     dfargs = list(df = object$df.residual)
     dffun = function(k, dfargs) dfargs$df
@@ -531,12 +591,14 @@ emm_basis.coxph = function (object, trms, xlev, grid, ...)
 {
     object$dist = "doesn't matter"
     result = emm_basis.survreg(object, trms, xlev, grid, ...)
-    result$dfargs$df = NA
-    result$X = result$X[, -1, drop = FALSE]
-    result$X = result$X - rep(object$means, each = nrow(result$X))
+    result$dfargs$df = Inf
+    nms = colnames(result$X)
+    # delete columns for intercept and main effects of strata
+    zaps = which(nms %in% setdiff(nms, names(result$bhat)))
+    result$X = result$X[, -zaps, drop = FALSE]
+    ### result$X = result$X - rep(object$means, each = nrow(result$X))
     result$misc$tran = "log"
     result$misc$inv.lbl = "hazard"
-    result$misc$postGridHook = .notran2   # removes "Surv()" as response transformation
     result
 }
 
@@ -600,7 +662,7 @@ emm_basis.coxme = function(object, trms, xlev, grid, ...) {
     V
 }
 
-# general-purpose emm_basis function
+# general-purpose emm_basis function for GEEs
 .emmb.geeGP = function(object, trms, xlev, grid, vcov.method, valid, idx = seq_along(valid), ...) {
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = object$contrasts)
@@ -623,14 +685,16 @@ emm_basis.coxme = function(object, trms, xlev, grid, ...) {
 ###  gee objects  ####
 
 
-recover_data.gee = recover_data.lm
+recover_data.gee = function(object, ...)
+    recover_data.lm(object, frame = NULL, ...)
 
 emm_basis.gee = function(object, trms, xlev, grid, vcov.method = "robust.variance", ...)
     .emmb.geeGP(object, trms, xlev, grid, vcov.method, 
                 valid = c("robust.variance", "naive.variance"))
 
 ###  geepack objects  ####
-recover_data.geeglm = recover_data.lm
+recover_data.geeglm = function(object, ...)
+    recover_data.lm(object, ...)
 
 emm_basis.geeglm = function(object, trms, xlev, grid, vcov.method = "vbeta", ...) {
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
@@ -693,35 +757,19 @@ emm_basis.geese = function(object, trms, xlev, grid, vcov.method = "vbeta", ...)
 }
 
 
+### survey pacvkage
+# svyglm class
+recover_data.svyglm = function(object, data = NULL, ...) {
+    if (is.null(data)) {
+        env = environment(terms(object))
+        des = eval(object$call$design, envir = env)
+        data = eval(des$call$data, envir = env)
+    }
+    recover_data.lm(object, data = data, frame = object$model, ...)
+}
 
+# inherited emm_basis.lm method works fine
 
-#--------------------------------------------------------------
-### glmmADMB package
-
-# recover_data.glmmadmb = recover_data.lm
-# 
-# emm_basis.glmmadmb = function (object, trms, xlev, grid, ...) 
-# {
-#     contrasts = attr(model.matrix(object), "contrasts")
-#     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
-#     X = model.matrix(trms, m, contrasts.arg = contrasts)
-#     bhat = glmmADMB::fixef(object)
-#     V = .my.vcov(object, ...)
-#     misc = list()
-#     if (!is.null(object$family)) {
-#         fam = object$family
-#         misc$tran = object$link
-#         misc$inv.lbl = "response"
-#         if (!is.na(pmatch(fam,"binomial"))) 
-#             misc$inv.lbl = "prob"
-#         else if (!is.na(pmatch(fam,"poisson"))) 
-#             misc$inv.lbl = "rate"
-#     }
-#     nbasis = estimability::all.estble
-#     dffun = function(...) Inf
-#     list(X = X, bhat = bhat, nbasis = nbasis, V = V, dffun = dffun, 
-#          dfargs = list(), misc = misc)
-# }
 
 
 
