@@ -30,26 +30,36 @@
 #' Add a grouping factor
 #' 
 #' This function adds a grouping factor to an existing reference grid or other 
-#' \code{emmGrid} object, such that the levels of an existing factor (call it the
-#' reference factor) are mapped to a smaller number of levels of the new
-#' grouping factor. The reference factor is then nested in the grouping factor. 
+#' \code{emmGrid} object, such that the levels of one or more existing factors (call them the
+#' reference factors) are mapped to a smaller number of levels of the new
+#' grouping factor. The reference factors are then nested in the new grouping factor. 
 #' This facilitates obtaining marginal means of the grouping factor, and 
 #' contrasts thereof.
 #'
 #' @param object An \code{emmGrid} object
 #' @param newname Character name of grouping factor to add (different from any
 #'   existing factor in the grid)
-#' @param refname Character name of the reference factor
-#' @param newlevs Character vector or factor of the same length as that of the levels for 
+#' @param refname Character name(s) of the reference factor(s)
+#' @param newlevs Character vector or factor of the same length as that of the (combined) levels for 
 #'   \code{refname}. The grouping factor \code{newname} will have the unique
-#'   values of \code{newlevs} as its levels.
+#'   values of \code{newlevs} as its levels. The order of levels in \code{newlevs}
+#'   is the same as the order of the level combinations produced by 
+#'   \code{\link{expand.grid}} applied to the levels of \code{refname} -- that is, the
+#'   first factor's levels change the fastest and the last one's vary the slowest.
 #'
 #' @return A revised \code{emmGrid} object having an additional factor named 
-#'   \code{newname}, and a new nesting structure \code{refname \%in\% newname}
+#'   \code{newname}, and a new nesting structure with each \code{refname \%in\% newname}
 #' 
 #' @note By default, the levels of \code{newname} will be ordered
 #'   alphabetically. To dictate a different ordering of levels, supply 
-#'   \code{newlevs} as a \code{factor} having its levels in the required order.
+#'   \code{newlevs} as a \code{factor} having its levels in the desired order.
+#'   
+#' @note When \code{refname} specifies more than one factor, this can
+#'   fundamentally (and permanently) change what is meant by the levels of those
+#'   individual factors. For instance, in the \code{gwrg} example below, there
+#'   are two levels of \code{wool} nested in each \code{prod}; and that implies
+#'   that we now regard these as four different kinds of wool. Similarly, there
+#'   are five different tensions (L, M, H in prod 1, and L, M in prod 2).
 #'   
 #' @export
 #'
@@ -64,12 +74,23 @@
 #' emmeans(gfrg, "machine")
 #' 
 #' emmeans(gfrg, "brand")
+#' 
+#' ### More than one reference factor
+#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
+#' gwrg <- add_grouping(ref_grid(warp.lm), 
+#'     "prod",  c("tension", "wool"),  c(2, 1, 1,  1, 2, 1))
+#'         # level combinations:         LA MA HA  LB MB HB
+#' 
+#' emmeans(gwrg, ~ wool * tension)   # some NAs due to impossible combinations
+#' 
+#' emmeans(gwrg, "prod")
+#' 
 add_grouping = function(object, newname, refname, newlevs) {
     if(!is.null(object@model.info$nesting[[refname]]))
         stop("'", refname, "' is already nested in another factor; cannot re-group it")
     if(newname %in% object@roles$predictors)
         stop("'", newname, "' is already the name of an existing predictor")
-    rlevs = object@levels[[refname]]
+    rlevs = do.call(paste, do.call(expand.grid, object@levels[refname]))
     if (length(newlevs) != length(rlevs))
         stop("Length of 'newlevs' doesn't match # levels of '", refname, "'")
     newlevs = factor(newlevs)
@@ -81,6 +102,7 @@ add_grouping = function(object, newname, refname, newlevs) {
     object@levels[[newname]] = glevs
     object@roles$predictors = c(object@roles$predictors, newname)
     
+    ref = do.call(paste, object@grid[refname]) # obs levels of rlevs
     wgt = object@grid$.wgt.
     if (is.null(wgt)) wgt = rep(1, nrow(object@grid))
     offset = object@grid$.offset.
@@ -94,7 +116,7 @@ add_grouping = function(object, newname, refname, newlevs) {
         g$.offset. = offset
         grid = rbind(grid, g)
         alevs = rlevs[newlevs == glevs[i]]
-        valid = c(valid, g[[refname]] %in% alevs)
+        valid = c(valid, ref %in% alevs)
     }
     # screen out invalid rows
     grid[!valid, ".wgt."] = 0
@@ -111,9 +133,10 @@ add_grouping = function(object, newname, refname, newlevs) {
     if (is.null(nesting))
         nesting = list()
     for (nm in names(nesting))
-        if (refname %in% nesting[[nm]])
+        if (any(refname %in% nesting[[nm]]))
             nesting[[nm]] = c(nesting[[nm]], newname)
-    nesting[[refname]] = newname
+    for (nm in refname)
+        nesting[[nm]] = newname   ### ??? should it be c(nesting[[nm]], newname)
     object@model.info$nesting = nesting
     
     object
@@ -137,6 +160,7 @@ add_grouping = function(object, newname, refname, newlevs) {
     #     if(!is.na(pmatch(dots$weights, "show.levels")))
     #         stop('weights = "show.levels" is not supported for nested models.')
 
+    orig.by = by   # save original 'by' vars
     #### Two issues to worry about....
     # (1) specs contains nested factors. We need to include their grouping factors
     xspecs = intersect(union(specs, by), names(nesting))
@@ -211,12 +235,19 @@ add_grouping = function(object, newname, refname, newlevs) {
     # preserve any nesting that still exists
     nesting = nesting[names(nesting) %in% names(result@levels)]
     result@model.info$nesting =   if (length(nesting) > 0) nesting    else NULL
+    
+    # resolve 'by'
+    by = orig.by
+    if (length(xspecs <- intersect(by, names(nesting))))
+        by = union(unlist(nesting[xspecs]), by)
+    result@misc$by.vars = by
+    
     result
 }
 
 
 ### contrast function for nested structures
-.nested_contrast = function(rgobj, method = "eff", by = NULL, adjust, ...) {
+.nested_contrast = function(rgobj, method = "eff", interaction = FALSE, by = NULL, adjust, ...) {
     nesting = rgobj@model.info$nesting
     # Prevent meaningless cases -- if A %in% B, we can't have A in 'by' without B
     # Our remedy will be to EXPAND the by list
@@ -225,47 +256,61 @@ add_grouping = function(object, newname, refname, newlevs) {
             by = union(by, nesting[[nm]])
             message("Note: Grouping factor(s) for '", nm, "' have been added to the 'by' list.")
         }
-    facs = setdiff(names(nesting), by)
-    if (length(facs) == 0)
-        stop("There are no factor levels left to contrast. Try taking nested factors out of 'by'.")
-    
+
     if(!is.character(method))
         stop ("Non-character contrast methods are not supported with nested objects")
     
-    testcon = get(paste0(method, ".emmc"))(1:3)
+    testcon = try(get(paste0(method, ".emmc"))(1:3), silent = TRUE)
+    if(inherits(testcon, "try-error")) testcon = NULL
     if(missing(adjust)) 
         adjust = attr(testcon, "adjust")
     estType = attr(testcon, "type")
 
     wkrg = rgobj # working copy
     facs = setdiff(names(wkrg@levels), by)  # these are the factors we'll combine & contrast
+    if (length(facs) == 0)
+        stop("There are no factor levels left to contrast. Try taking nested factors out of 'by'.")
     if (!is.null(display <- wkrg@misc$display))
         wkrg = wkrg[which(display), drop.levels = TRUE]
     wkrg@model.info$nesting = wkrg@misc$display = NULL
     by.rows = .find.by.rows(wkrg@grid, by)
     if(length(by.rows) == 1)
-        result = contrast(wkrg, method = method, by = by, ...)
+        result = contrast.emmGrid(wkrg, method = method, interaction = interaction, by = by, adjust = adjust, ...)
     else {
         result = lapply(by.rows, function(rows) {
             contrast.emmGrid(wkrg[rows, drop.levels = TRUE], method = method, 
-                              by = by, adjust = adjust, ...)
+                             interaction = interaction, by = by, adjust = adjust, ...)
         })
+        # set up coef matrix
+        comb.nms = unique(do.call(paste, wkrg@grid[facs]))
+        ncon = sapply(result, function(x) nrow(x@grid))
+        con.idx = rep(seq_along(by.rows), ncon)  ## looks like 1,1, 2,2,2, 3, 4,4 ...
+        con.coef = matrix(0, nrow = sum(ncon), ncol = nrow(wkrg@grid))
         # Have to define .wgt. for nested emmGrid. Use average weight - seems most sensible
-        for (i in seq_along(by.rows))
+        for (i in seq_along(by.rows)) {
             result[[i]]@grid$.wgt. = mean(wkrg@grid[[".wgt."]][by.rows[[i]]])
+            con.coef[con.idx == i, by.rows[[i]]] = result[[i]]@misc$con.coef
+        }
         result$adjust = ifelse(is.null(adjust), "none", adjust)
         result = do.call(rbind.emmGrid, result)
+        result@misc$con.coef = con.coef
+        result@misc$orig.grid = wkrg@grid[names(wkrg@levels)]
         result = update(result, by = by, 
                         estType = ifelse(is.null(estType), "contrast", estType))
         cname = setdiff(names(result@levels), by)
-        result@model.info$nesting[[cname]] = by
+        if (!is.null(result@model.info$nesting))
+            for (nm in cname) 
+                result@model.info$nesting[[nm]] = by
     }
-    result@misc$orig.grid = result@misc$con.code = NULL
+##    result@misc$orig.grid = result@misc$con.coef = NULL # we now provide these
 
     for (nm in by) {
         if (nm %in% names(nesting))
             result@model.info$nesting[[nm]] = intersect(nesting[[nm]], by)
     }
+    if(!is.na(ET <- result@misc$estType) && (ET == "pairs")) # internal flag to keep track of original by vars for paired comps
+        result@misc$.pairby = paste(c("", by), collapse = ",")
+    
     result
 }
 
@@ -293,6 +338,50 @@ add_grouping = function(object, newname, refname, newlevs) {
     keep
 }
 
+# Fill-in extra elements to make a grid regular
+#' @rdname rbind.emmGrid
+#' @order 9
+#' @param object an object of class \code{emmGrid}
+#' @return \code{force_regular} adds extra (invisible) rows to an \code{emmGrid} object
+#'   to make it a regular grid (all combinations of factors). This regular structure is 
+#'   needed by \code{emmeans}. An object can become irregular by, for example,
+#'   subsetting rows, or by obtaining contrasts of a nested structure.
+#' @export
+#' @examples
+#' 
+#' ### Irregular object
+#' tmp <- warp.rg[-1]
+#' ## emmeans(tmp, "tension")   # will fail because tmp is irregular
+#' emmeans(force_regular(tmp), "tension")   # will show some results
+force_regular = function(object) {
+    newgrid = do.call(expand.grid, object@levels)
+    newkey = do.call(paste, newgrid)
+    newlf = matrix(NA, nrow = nrow(newgrid), ncol = ncol(object@linfct))
+    colnames(newlf) = colnames(object@linfct) 
+    newdisp = rep(FALSE, nrow(newgrid))
+    
+    oldgrid = object@grid
+    oldkey = do.call(paste, oldgrid[setdiff(names(oldgrid), c(".wgt.", ".offset."))])
+    if (wtd <- (".wgt." %in% names(oldgrid))) 
+        newgrid$.wgt. = 0
+    if (ofs <- (".offset." %in% names(oldgrid))) 
+        newgrid$.offset. = NA
+    for (j in seq_along(oldkey)) {
+        key = oldkey[j]
+        i = which(newkey == key)
+        newlf[i, ] = object@linfct[j, ]
+        newdisp[i] = TRUE
+        if(wtd) newgrid$.wgt.[i] = oldgrid$.wgt.[j]
+        if(ofs) newgrid$.offset.[i] = oldgrid$.offset.[j]
+    }
+    object@grid = newgrid
+    object@linfct = newlf
+    if(is.null(object@model.info$nesting) && all(newdisp))  # remove 'display` if all TRUE`
+        newdisp = NULL
+    object@misc$display = newdisp
+    object
+}
+
 
 # Internal function to find nesting
 # We look at two things:
@@ -304,6 +393,8 @@ add_grouping = function(object, newname, refname, newlevs) {
 #     trms$factors
 # The function returns a named list, e.g., list(A = "B")
 # If none found, an empty list is returned.
+# 
+# Added ver 1.5.3+ : if trms is NULL, we skip that part
 .find_nests = function(grid, trms, coerce, levels) {
     result = list()
     
@@ -326,47 +417,63 @@ add_grouping = function(object, newname, refname, newlevs) {
             result[[nm]] = otrs[max.levs == 1]
     }
     
-    # Now look at factors attribute
-    fac = attr(trms, "factors")
-    if (length(fac) > 0) {
-        if (!is.null(coerce)) for (stg in coerce) {
-            subst = paste(.all.vars(stats::reformulate(stg)), collapse = ":")
-            for (i in 1:2)
-                dimnames(fac)[[i]] = gsub(stg, subst, dimnames(fac)[[i]], 
-                                          fixed = TRUE)
-        }
-        fac = fac[intersect(nms, row.names(fac)), , drop = FALSE]
+    if(!is.null(trms)) {
         
-        ### new code
-        nms = row.names(fac)
-        cols = dimnames(fac)[[2]]
-        pert = setdiff(nms, intersect(nms, cols)) # pertinent - no main effect in model
-        for (nm in pert) {
-            pfac = fac[, fac[nm, ] == 1, drop = FALSE]  # cols where nm appears
-            if (ncol(pfac) == 0) { # case where there is no 1 in a row
-                pfac = fac[, fac[nm, ] == 2, drop = FALSE]
-                pfac[nm, ] = 1 # make own entry 1 so it isn't nested in self
+        # Now look at factors attribute
+        fac = attr(trms, "factors")
+        if (length(fac) > 0) {
+            if (!is.null(coerce)) for (stg in coerce) {
+                subst = paste(.all.vars(stats::reformulate(stg)), collapse = ":")
+                for (i in 1:2)
+                    dimnames(fac)[[i]] = gsub(stg, subst, dimnames(fac)[[i]], 
+                                              fixed = TRUE)
+            }
+            fac = fac[intersect(nms, row.names(fac)), , drop = FALSE]
+            
+            ### new code
+            nms = row.names(fac)
+            cols = dimnames(fac)[[2]]
+            pert = setdiff(nms, intersect(nms, cols)) # pertinent - no main effect in model
+            for (nm in pert) {
+                pfac = fac[, fac[nm, ] == 1, drop = FALSE]  # cols where nm appears
+                if (ncol(pfac) == 0) { # case where there is no 1 in a row
+                    pfac = fac[, fac[nm, ] == 2, drop = FALSE]
+                    pfac[nm, ] = 1 # make own entry 1 so it isn't nested in self
+                }
+                nst = .strip.supersets(apply(pfac, 2, function(col) nms[col == 2]))
+                if (length(nst) > 0)
+                    result[[nm]] = union(result[[nm]], nst)
             }
-            nst = unique(as.character(apply(pfac, 2, function(col) nms[col == 2])))
-            if (length(nst) > 0)
-                result[[nm]] = union(result[[nm]], nst)
         }
-        ### old code -- included some specious results
-        # for (j in seq_len(ncol(fac))) {
-        #     if (any(fac[, j] == 2)) {
-        #         nst = nms[fac[, j] == 1]
-        #         for (nm in nst)
-        #             result[[nm]] = nms[fac[, j] == 2]
-        #     }
-        # }
-    }
+        
+    }  # [end if(!is.null(trms))]
+    
     # include nesting factors that are themselves nested
     for (nm in names(result))
         result[[nm]] = union(unlist(result[result[[nm]]]), result[[nm]])
+
+    # omit any entries where factors are nested in themselves (we get such in a model like y ~ A : B)
+    for (nm in names(result))
+        if (nm %in% result[[nm]])
+            result[[nm]] = NULL
+    
     
     result
 }
 
+# strip supersets from a list and condense down to a character vector
+# e.g., lst = list("a", c("A", "B")) --> "A"
+.strip.supersets = function(lst) {
+    if (is.list(lst) && (length(lst) > 1)) {
+        lst = lst[order(sapply(lst, length))]  # order by length
+        for (i in length(lst):2) {
+            tst = sapply(lst[1:(i-1)], function(x) all(x %in% lst[[i]]))
+            if (any(tst)) lst[[i]] = NULL
+        }
+    }
+    unique(unlist(lst))
+}
+
 # internal function to format a list of nested levels
 .fmt.nest = function(nlist) {
     if (length(nlist) == 0)
