@@ -1,5 +1,5 @@
 ##############################################################################
-#    Copyright (c) 2012-2019 Russell V. Lenth                                #
+#    Copyright (c) 2012-2020 Russell V. Lenth                                #
 #                                                                            #
 #    This file is part of the emmeans package for R (*emmeans*)              #
 #                                                                            #
@@ -85,7 +85,7 @@
 #' @param type Character value. If provided, this is saved as the
 #'   \code{"predict.type"} setting. See \code{\link{update.emmGrid}} and the
 #'   section below on prediction types and transformations.
-#' @param transform Character value. If other than \code{"none"}, the reference
+#' @param transform Character, logical, or list. If non-missing, the reference
 #'   grid is reconstructed via \code{\link{regrid}} with the given
 #'   \code{transform} argument. See the section below on prediction types and
 #'   transformations.
@@ -102,7 +102,19 @@
 #' @param sigma Numeric value to use for subsequent predictions or
 #'   back-transformation bias adjustments. If not specified, we use
 #'   \code{sigma(object)}, if available, and \code{NULL} otherwise.
-#' @param ... Optional arguments passed to \code{\link{emm_basis}}, and
+#' @param nuisance,non.nuisance,wt.nuis If \code{nuisance} is a vector of predictor names,
+#'   those predictors are omitted from the reference grid. Instead, the result 
+#'   will be as if we had averaged over the levels of those factors, with either 
+#'   equal or proportional weights as specified in \code{wt.nuis} (see the 
+#'   \code{weights} argument in \code{\link{emmeans}}). The factors in 
+#'   \code{nuisance} must not interact with other factors, not even other
+#'   nuisance factors. Specifying nuisance factors can save considerable
+#'   storage and computation time, and help avoid exceeding the maximum
+#'   reference-grid size (\code{get_emm_option("rg.limit")}).
+#' @param rg.limit Integer limit on the number of reference-grid rows to allow
+#'   (checked before any multivariate responses are included).
+#' @param ... Optional arguments passed to \code{\link{summary.emmGrid}},
+#'   \code{\link{emm_basis}}, and
 #'   \code{\link{recover_data}}, such as \code{params}, \code{vcov.} (see
 #'   \bold{Covariance matrix} below), or options such as \code{mode} for
 #'   specific model types (see \href{../doc/models.html}{vignette("models",
@@ -140,6 +152,17 @@
 #'   with the reference grid as \code{newdata}. (This is done \emph{after} the
 #'   reference grid is determined.) A formula is appropriate here when you think
 #'   experimental conditions affect the covariate as well as the response.
+#'   
+#'   To allow for situations where a simple \code{lm()} call as described above won't
+#'   be adequate, a formula of the form \code{ext ~ fcnname} is also supported,
+#'   where the left-hand side may be \code{ext}, \code{extern}, or
+#'   \code{external} (and must \emph{not} be a predictor name) and the
+#'   right-hand side is the name of an existing function. The function is called
+#'   with one argument, a data frame with columns for each variable in the
+#'   reference grid. The function is expected to use that frame as new data to
+#'   be used to obtain predictions for one or more models; and it should return
+#'   a named list or data frame with replacement values for one or more of the
+#'   covariates.
 #'
 #'   If \code{cov.reduce} is a named list, then the above criteria are used to
 #'   determine what to do with covariates named in the list. (However, formula
@@ -195,7 +218,10 @@
 #'   \code{vcov.} argument, specifying a matrix or a function. If a matrix, it
 #'   must be square and of the same dimension and parameter order of the fixed
 #'   effects. If a function, must return a suitable matrix when it is called
-#'   with \code{object} as its only argument.
+#'   with arguments \code{(object, ...)}. Be careful with possible 
+#'   unintended conflicts with arguments in \code{...}; for example, 
+#'   \code{sandwich::vcovHAC()} has optional arguments \code{adjust} and \code{weights}
+#'   that may be intended for \code{emmeans()} but will also be passed to \code{vcov.()}.
 #'
 #'   \bold{Nested factors.} Having a nesting structure affects marginal
 #'   averaging in \code{emmeans} in that it is done separately for each level
@@ -262,14 +288,15 @@
 #'   \samp{transform = "log", type = "response"}. See also the help page for 
 #'   \code{\link{regrid}}.
 #'
-#' @section Side effect: The most recent result of \code{ref_grid}, whether
+#' @section Optional side effect: If the \code{save.ref_grid} option is set to
+#'   \code{TRUE} (see \code{\link{emm_options}}),
+#'   The most recent result of \code{ref_grid}, whether
 #'   called directly or indirectly via \code{\link{emmeans}},
 #'   \code{\link{emtrends}}, or some other function that calls one of these, is
 #'   saved in the user's environment as \code{.Last.ref_grid}. This facilitates
 #'   checking what reference grid was used, or reusing the same reference grid
-#'   for further calculations. This automatic saving is enabled by default, but
-#'   may be disabled via \samp{emm_options(save.ref_grid = FALSE)}, and
-#'   re-enabled by specifying \code{TRUE}.
+#'   for further calculations. This automatic saving is disabled by default, but
+#'   may be enabled via \samp{emm_options(save.ref_grid = TRUE)}.
 #'
 #' @return An object of the S4 class \code{"emmGrid"} (see
 #'   \code{\link{emmGrid-class}}). These objects encapsulate everything needed
@@ -281,6 +308,8 @@
 #'   \code{\link{summary.emmGrid}}. Reference grids are fundamental to
 #'   \code{\link{emmeans}}. Supported models are detailed in
 #'   \href{../doc/models.html}{\code{vignette("models", "emmeans")}}.
+#'   See \code{\link{update.emmGrid}} for details of arguments that can be in 
+#'   \code{options} (or in \code{...}).
 #'   
 #' @note The system default for \code{cov.keep} causes models
 #'   containing indicator variables to be handled differently than in
@@ -298,7 +327,6 @@
 #' @examples
 #' fiber.lm <- lm(strength ~ machine*diameter, data = fiber)
 #' ref_grid(fiber.lm)
-#' summary(.Last.ref_grid)
 #'
 #' ref_grid(fiber.lm, at = list(diameter = c(15, 25)))
 #'
@@ -321,6 +349,21 @@
 #' # Two of these have the same grid but different weights:
 #' rg.default@grid
 #' rg.at@grid
+#' 
+#' ### Using cov.reduce formulas...
+#' # Above suggests we can vary disp indep. of other factors - unrealistic
+#' rg.alt <- ref_grid(mtcars.lm, at = list(wt = c(2.5, 3, 3.5)),
+#'     cov.reduce = disp ~ vs * wt)
+#' rg.alt@grid
+#' 
+#' # Alternative to above where we model sqrt(disp)
+#' disp.mod <- lm(sqrt(disp) ~ vs * wt, data = mtcars)
+#' disp.fun <- function(dat)
+#'     list(disp = predict(disp.mod, newdata = dat)^2)
+#' rg.alt2 <- ref_grid(mtcars.lm, at = list(wt = c(2.5, 3, 3.5)),
+#'     cov.reduce = external ~ disp.fun)
+#' rg.alt2@grid
+#' 
 #'
 #' # Multivariate example
 #' MOats.lm = lm(yield ~ Block + Variety, data = MOats)
@@ -328,13 +371,21 @@
 #' # Silly illustration of how to use 'mult.levs' to make comb's of two factors
 #' ref_grid(MOats.lm, mult.levs = list(T=LETTERS[1:2], U=letters[1:2]))
 #' 
+#' # Using 'params'
+#' require("splines")
+#' my.knots = c(2.5, 3, 3.5)
+#' mod = lm(Sepal.Length ~ Species * ns(Sepal.Width, knots = my.knots), data = iris)
+#' ## my.knots is not a predictor, so need to name it in 'params'
+#' ref_grid(mod, params = "my.knots") 
+#' 
 ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("cov.keep"),
                      mult.names, mult.levs, 
                      options = get_emm_option("ref_grid"), data, df, type, 
-                     transform = c("none", "response", "mu", "unlink", "log"), 
-                     nesting, offset, sigma, ...) 
+                     transform, nesting, offset, sigma, 
+                     nuisance = character(0), non.nuisance, wt.nuis = "equal", 
+                     rg.limit = get_emm_option("rg.limit"), ...) 
 {
-    transform = match.arg(transform)
+    ### transform = match.arg(transform)
     if (!missing(df)) {
         if(is.null(options)) options = list()
         options$df = df
@@ -440,6 +491,8 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         if (is.factor(x) && !(nm %in% coerced$covariates))
             xlev[[nm]] = levels(factor(x))
             # (applying factor drops any unused levels)
+        else if (is.character(x)) # just like a factor
+            xlev[[nm]] = sort(unique(x))
     
         # Now go thru and find reference levels...
         # mentioned in 'at' list but not coerced factor
@@ -478,8 +531,18 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     }
     
 
+    if (!missing(non.nuisance))
+        nuisance = setdiff(names(ref.levels), non.nuisance)
+
     # Now create the reference grid
-    grid = do.call(expand.grid, ref.levels)
+    if(no.nuis <- (length(nuisance) == 0)) {
+        .check.grid(ref.levels, rg.limit)
+        grid = do.call(expand.grid, ref.levels)
+    }
+    else {
+        nuis.info = .setup.nuis(nuisance, ref.levels, trms, rg.limit)
+        grid = nuis.info$grid
+    }
     
     # undocumented hook to expand grid by increments of 'var' (needed by emtrends)
     if (!is.null(delts <- options$delts)) {
@@ -487,6 +550,8 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         n.orig = nrow(grid) # remember how many rows we had
         grid = grid[rep(seq_len(n.orig), length(delts)), , drop = FALSE]
         options$var = options$delts = NULL
+        # grid[[var]] = grid[[var]] + rep(delts, each = n.orig)
+        #     (we have to wait until after covariate calcs to do this)
     }
     
     # add any matrices
@@ -498,39 +563,84 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
 
     # resolve any covariate formulas
     for (xnm in names(dep.x)) {
-        if (!all(.all.vars(dep.x[[xnm]]) %in% names(grid)))
+        if ((xnm %in% c("ext", "extern", "external")) && !(xnm %in% names(grid))) { # use external fcn
+            fun = get(as.character(dep.x[[xnm]][[3]]), inherits = TRUE)
+            rslts = fun(grid)   # should be some thing that supports names() and [[]], e.g., a list or d.f.
+            for (nm in  intersect(names(rslts), names(grid))) {
+                grid[[nm]] = rslts[[nm]]
+                ref.levels[[nm]] = NULL
+            }
+        }
+        else if (!all(.all.vars(dep.x[[xnm]]) %in% names(grid)))
             stop("Formulas in 'cov.reduce' must predict covariates actually in the model")
-        xmod = lm(dep.x[[xnm]], data = data)
-        grid[[xnm]] = predict(xmod, newdata = grid)
-        ref.levels[[xnm]] = NULL
+        else { # Use lm() to predict this covariate
+            xmod = lm(dep.x[[xnm]], data = data)
+            grid[[xnm]] = predict(xmod, newdata = grid)
+            ref.levels[[xnm]] = NULL
+        }
     }
     
     # finish-up our hook for expanding the grid
     if (!is.null(delts)) # add increments if any
         grid[[var]] = grid[[var]] + rep(delts, each = n.orig)
     
-    if (!is.null(attr(data, "pass.it.on")))   # a hook needed by emm_basis.gamlss
+    if (!is.null(attr(data, "pass.it.on")))   # a hook needed by emm_basis.gamlss et al.
         attr(object, "data") = data
     
+    ###!! Prevent a warning like in https://stackoverflow.com/questions/68969384/emmeans-warning-in-model-frame-defaultformula-data-data-variable-gr/68990172#68990172
+    xl = xlev
+    modnm = rownames(attr(trms, "factors"))
+    chk = sapply(modnm, function(mn) mn %in% names(xl))
+    for(i in which(!chk)) { # replace names in xl - e.g., as.factor(trt) where trt already a factor
+        fn = all.vars(reformulate(modnm[i]))
+        if (length(fn) == 1)
+            names(xl)[names(xl) == fn] = modnm[i]
+    }
+    ###!! If we remove this code, also need to change xl back to xlev in 'basis =' call below
+    
     # we've added args `misc` and `options` so emm_basis methods can access and use these if they want
-    basis = emm_basis(object, trms, xlev, grid, misc = attr(data, "misc"), options = options, ...)
+    basis = emm_basis(object, trms, xl, grid, misc = attr(data, "misc"), options = options, ...)
+    environment(basis$dffun) = baseenv()   # releases unnecessary storage
     if(length(basis$bhat) != ncol(basis$X))
-        stop("Non-conformable elements in reference grid.\n",
-             " Probably due to rank deficiency not handled as expected.",
+        stop("Something went wrong:\n",
+             " Non-conformable elements in reference grid.",
              call. = TRUE)
     
+    if(!no.nuis) {
+        basis = .basis.nuis(basis, nuis.info, wt.nuis, ref.levels, data, grid, ref.levels)
+        grid = basis$grid
+        nuisance = ref.levels[nuis.info$nuis] # now nuisance has the levels info
+        ref.levels = basis$ref.levels
+    }
+    
     misc = basis$misc
     
     ### Figure out if there is a response transformation...
     # next stmt assumes that model formula is 1st argument (2nd element) in call.
     # if not, we probably get an error or something that isn't a formula
     # and it is silently ignored
-    lhs = try(eval(as.formula(attr(data, "call")[[2]])[-3]), silent = TRUE)
-    if (inherits(lhs, "formula")) { # response may be transformed
+    frm = try(formula(eval(attr(data, "call")[[2]])), silent = TRUE)
+    if (inherits(frm, "formula")) { # response may be transformed
+        lhs = frm[-3]
         tran = setdiff(.all.vars(lhs, functions = TRUE), c(.all.vars(lhs), "~", "cbind", "+", "-", "*", "/", "^", "%%", "%/%"))
         if(length(tran) > 0) {
-            if (tran[1] == "linkfun")
-                tran = as.list(environment(trms))
+            if (tran[1] == "scale") { # we'll try to set it up based on terms component
+                pv = try(attr(terms(object), "predvars"), silent = TRUE)
+                if (!inherits(pv, "try-error")) {
+                    pv = c(lapply(pv, as.character), "foo") # make sure it isn't empty
+                    scal = which(sapply(pv, function(x) x[1] == "scale"))
+                    if(length(scal) > 0) {
+                        par = as.numeric(pv[[scal[1]]][3:4]) 
+                        tran = make.tran("scale", y = 0, center = par[1], scale = par[2])
+                    }
+                }
+                if (is.character(tran)) { # didn't manage to find params
+                    tran = NULL
+                    message("NOTE: Unable to recover scale() parameters. See '? make.tran'")
+                }
+            }
+            else if (tran[1] == "linkfun")
+                tran = as.list(environment(trms))[c("linkfun","linkinv","mu.eta","valideta","name")]
             else {
                 if (tran[1] == "I") 
                     tran = "identity"
@@ -589,7 +699,6 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         multresp = names(mult.levs)
         MF = do.call("expand.grid", mult.levs)
         grid = merge(grid, MF)
-        
     }
             
     # add any matrices
@@ -644,6 +753,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         data[["(weights)"]] = 1
     cov.keep = intersect(unique(cov.keep), names(ref.levels))
     nms = union(union(union(names(xlev), names(chrlev)), coerced$factors), cov.keep)
+    nms = intersect(nms, names(grid))
     #### Old code...
     # if (!covnest)
     #     nms = union(union(names(xlev), names(chrlev)), coerced$factors) # only factors, no covariates or mult.resp
@@ -652,7 +762,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     if (length(nms) == 0)
         wgt = rep(1, nrow(grid))  # all covariates; give each weight 1
     else {
-        id = plyr::id(data[, nms, drop = FALSE], drop = TRUE)
+        id = .my.id(data[, nms, drop = FALSE])
         uid = !duplicated(id)
         key = do.call(paste, unname(data[uid, nms, drop = FALSE]))
         key = key[order(id[uid])]
@@ -664,6 +774,10 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     grid[[".wgt."]] = wgt
     
     model.info = list(call = attr(data,"call"), terms = trms, xlev = xlev)
+    if (!is.null(mm <- basis$model.matrix)) { # submodel support
+        attr(mm, "factors") = .smpFT(trms)
+        model.info$model.matrix = mm  
+    }
     # Detect any nesting structures
     nst = .find_nests(grid, trms, coerced$orig, ref.levels)
     if (length(nst) > 0)
@@ -677,14 +791,15 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     misc$level = .95
     misc$adjust = "none"
     misc$famSize = nrow(grid)
-    misc$avgd.over = character(0)
+    if(is.null(misc$avgd.over))
+       misc$avgd.over = character(0)
     misc$sigma = sigma
 
     post.beta = basis$post.beta
     if (is.null(post.beta))
         post.beta = matrix(NA)
     
-    predictors = attr(data, "predictors")
+    predictors = intersect(attr(data, "predictors"), names(grid))
     
     simp.tbl = environment(trms)$.simplify.names.
     if (! is.null(simp.tbl)) {
@@ -704,7 +819,8 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
          model.info = model.info,
          roles = list(predictors = predictors, 
                       responses = attr(data, "responses"), 
-                      multresp = multresp),
+                      multresp = multresp,
+                      nuisance = nuisance),
          grid = grid, levels = ref.levels, matlevs = matlevs,
          linfct = basis$X, bhat = basis$bhat, nbasis = basis$nbasis, V = basis$V,
          dffun = basis$dffun, dfargs = basis$dfargs, 
@@ -732,10 +848,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
                     "fitted model:\n    ", .fmt.nest(nst))
     }
 
-    if(!is.null(options)) {
-        options$object = result
-        result = do.call("update.emmGrid", options)
-    }
+    result = .update.options(result, options, ...)
 
     if(!is.null(hook <- misc$postGridHook)) {
         if (is.character(hook))
@@ -743,7 +856,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         result@misc$postGridHook = NULL
         result = hook(result, ...)
     }
-    if(transform != "none")
+    if(!missing(transform))
         result = regrid(result, transform = transform, sigma = sigma, ...)
     
     .save.ref_grid(result)
@@ -803,5 +916,112 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     list(factors = cfac, covariates = ccov, orig = orig)
 }
 
+# My replacement for plyr::id(, drop = TRUE)
+.my.id = function(data){
+    p = do.call(paste, data)
+    u = unique(p)
+    match(p, u)
+}
 
+# Utility to error-out when potential reference grid size is too big
+.check.grid = function(levs, limit = get_emm_option("rg.limit")) {
+    size = prod(sapply(levs, length))
+    if (size > limit)
+        stop("The rows of your requested reference grid would be ", size, ", which exceeds\n",
+             "the limit of ", limit, " (not including any multivariate responses).\n",
+             "Your options are:\n",
+             "  1. Specify some (or more) nuisance factors using the 'nuisance' argument\n",
+             "     (see ?ref_grid). These must be factors that do not interact with others.\n",
+             "  2. Add the argument 'rg.limit = <new limit>' to the call. Be careful,\n",
+             "     because this could cause excessive memory use and performance issues.\n",
+             "     Or, change the default via 'emm_options(rg.limit = <new limit>)'.\n",
+             call. = FALSE)
+}
+
+# Utility to set up the grid for nuisance factors. This consists of two or more
+# data.frames rbinded together:
+#   * the expanded grid for all factors *not* in nuis, with the nuis factors set 
+#     at their first levels
+#   * for each factor in f nuis, a set of rows for (levs$f), with the other 
+#     factors at their first levels (this is arbitrary)
+# In addition, we return a character vector 'row.assign' corresponding to the rows 
+# in the grid, telling which is what: ".main.grid." for the first part of the grid,
+# otherwise factor names from nuis.
+# We also return 'nuis' itself - which may be reduced since illegal 
+# entries are silently removed
+.setup.nuis = function(nuis, levs, trms, rg.limit) {
+    firsts =  args = lapply(levs, function(x) x[1])
+    nuis = intersect(nuis, names(levs))
+    # sanity checks on terms, and term indexes
+    fsum = rep(99, length(nuis))
+    tbl = attr(trms, "factors")
+    rn = row.names(tbl) = sapply(row.names(tbl), function(nm)
+        paste(all.vars(reformulate(nm)), collapse = ","))
+    for (i in seq_along(nuis)) {
+        f = nuis[i]
+        if(f %in% rn)
+            fsum[i] = sum(tbl[f, ])
+    }
+    nuis = nuis[fsum == 1]   # silently remove any unfound or interacting factors
+
+    # top part...
+    non.nuis = setdiff(names(levs), nuis)
+    for (n in non.nuis)
+        args[[n]] = levs[[n]]
+    .check.grid(args, rg.limit)
+    grid = do.call(expand.grid, args)
+    ra = rep(".main.grid.", nrow(grid))
+    # bottom parts
+    for (f in nuis) {
+        args = firsts
+        args[[f]] = levs[[f]]
+        grid = rbind(grid, do.call(expand.grid, args))
+        ra = c(ra, rep(f, length(levs[[f]])))
+    }
+    list(grid = grid, row.assign = ra, nuis = nuis)
+}
+
+# Do the required post-processing for nuisance factors...
+# After we get the model matrix for this grid, we'll average each set of rows in the
+# bottom part, and substitute those averages in the required columns in the top part 
+# of the model matrix.
+.basis.nuis = function(basis, info, wt, levs, data, grid, ref.levels) {
+    ra = info$row.assign
+    r. = rep(".", length(ra))  # fillers
+    X = basis$X
+    n = sum(ra == ".main.grid.")
+    k = nrow(X) / length(ra)   # multivariate dimension
+    nuis = info$nuis
+    wts = lapply(nuis, function(f) {
+        if (wt == "equal")
+            w = rep(1, length(levs[[f]]))
+        else {
+            x = data[[f]]
+            w = sapply(levs[[f]], function(lev) sum(x == lev))
+        }
+        w / sum(w)
+    })
+    names(wts) = nuis
+    
+    # In a multivariate case, we have to repeat the same operations for each block of X rows
+    for (m in 1:k) {
+        RA = c(rep(r., m - 1), ra, rep(r., k - m))
+        for (f in nuis) {
+            subX = X[RA == f, , drop = FALSE]
+            cols = which(apply(subX, 2, function(x) diff(range(x)) > 0))
+            subX = sweep(subX[, cols, drop = FALSE], 1, wts[[f]], "*")
+            avg = apply(subX, 2, sum)
+            avg = matrix(rep(avg, each = n), nrow = n) # now several copies
+            X[RA == ".main.grid.", cols] = avg
+        }
+    }
+    basis$misc$nuis = nuis
+    basis$misc$avgd.over = paste(length(nuis), "nuisance factors")
+    RA = rep(ra, k)
+    basis$X = X[RA == ".main.grid.", , drop = FALSE]
+    non.nuis = setdiff(names(ref.levels), info$nuis)
+    basis$ref.levels = ref.levels[non.nuis]
+    basis$grid = grid[1:n, non.nuis, drop = FALSE]
+    basis
+}
 
