@@ -213,6 +213,10 @@
 #'   one-step adjustment, but it can be very slow. One may use
 #'   \code{\link{as.glht}} with methods in the \pkg{multcomp} package to obtain
 #'   non-conservative multi-step adjustments to tests.
+#'   
+#'   \emph{Warning:} Non-estimable cases are \emph{included} in the family to which adjustments
+#'   are applied. You may wish to subset the object using the \code{[]} operator
+#'   to work around this problem.
 #'
 #' @section Tests of significance, nonsuperiority, noninferiority, or equivalence:
 #'   When \code{delta = 0}, test statistics are the usual tests of significance.
@@ -262,11 +266,14 @@
 #'   \code{1e-8}. You may change it via \code{\link{emm_options}} by setting
 #'   \code{estble.tol} to the desired value.
 #'   
+#'   See the warning above that non-estimable cases are still included when
+#'   determining the family size for \emph{P}-value adjustments.
+#'   
 #' @section Warning about potential misuse of P values:
-#'   A growing consensus in the statistical and scientific community is that
+#'   Some in the statistical and scientific community argue that
 #'   the term \dQuote{statistical significance} should be completely abandoned, and
 #'   that criteria such as \dQuote{p < 0.05} never be used to assess the
-#'   importance of an effect. These practices are just too misleading and prone to abuse.
+#'   importance of an effect. These practices can be too misleading and are prone to abuse.
 #'   See \href{../doc/basics.html#pvalues}{the \dQuote{basics} vignette} for more
 #'   discussion.
 #'   
@@ -276,8 +283,14 @@
 #'   scale of the linear predictor, regardless of the setting for `type`. If
 #'   \code{type = "response"}, the null value displayed in the summary table 
 #'   will be back-transformed from the value supplied by the user. But the
-#'   displayed \code{delta} will not be changed, because there (usually) is
+#'   displayed \code{delta} will not be changed, because there (often) is
 #'   not a natural way to back-transform it.
+#'   
+#' @note When we have \code{type = "response"}, and \code{bias.adj = TRUE},
+#'   the \code{null} value displayed in the output is both back-transformed
+#'   and bias-adjusted, leading to a rather non-intuitive-looking null value.
+#'   However, since the tests themselves are performed on the link scale,
+#'   this is the response value at which a *P* value of 1 would be obtained.
 #' 
 #' @note The default \code{show} method for \code{emmGrid} objects (with the
 #'   exception of newly created reference grids) is \code{print(summary())}.
@@ -362,14 +375,30 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
     if(missing(by))
         by = misc$by.vars
     
+    # Disable Tukey if by vars don't match those used in construction
+    if((!is.na(misc$estType) && misc$estType == "pairs") && (paste(c("", by), collapse = ",") != misc$.pairby))
+        misc$estType = object@misc$estType = "contrast"
+    
+    
+    
     if (missing(type))
         type = .get.predict.type(misc)
     else
         type = .validate.type(type)
     
     # if there are two transformations and we want response, then we need to undo both
-    if ((type == "response") && (!is.null(misc$tran2)))
+    if ((type == "response") && (!is.null(misc$tran2))) {
+        tmp = match.call()
+        tmp$type = "unlink"
+        summ.unlink = eval(tmp) #this is summary with type = "unlink"
         object = regrid(object, transform = "mu")
+        two.trans = TRUE
+    }
+    else
+        two.trans = FALSE
+    
+
+    
     if ((type %in% c("mu", "unlink")) && (!is.null(t2 <- misc$tran2))) {
         if (!is.character(t2))
             t2 = "tran"
@@ -470,14 +499,16 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
     fam.info = c(misc$famSize, by.size, et)
     cnm = NULL
     adjust = tolower(adjust)
+    if (adjust %in% c("bh", "by"))  # special cases in p.adjust.methods
+        adjust = toupper(adjust)
     
     # get vcov matrix only if needed (adjust == "mvt")
     corrmat = sch.rank = NULL
-    if (!is.na(pmatch(adjust, "mvt"))) {
+    if (adjust %.pin% "mvt") { ##(!is.na(pmatch(adjust, "mvt"))) {
         corrmat = cov2cor(vcov(object))
         attr(corrmat, "by.rows") = by.rows
     }
-    else if (!is.na(pmatch(adjust, "scheffe"))) {
+    else if (adjust %.pin% "scheffe") {  ##(!is.na(pmatch(adjust, "scheffe"))) {
         if(is.null(sch.rank <- .match.dots("scheffe.rank", ...)))
             sch.rank = sapply(by.rows, function(.) qr(zapsmall(object@linfct[., , drop = FALSE]))$rank)
         if(length(unique(sch.rank)) > 1)
@@ -497,7 +528,14 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
                 " so it is omitted")
         }
     }
-
+    
+    
+    # add linkname attribute
+    if (two.trans)
+        linkname = paste0(link$name, "[", attr(summ.unlink, "linkname"), "]")
+    else
+        linkname = link$name
+    
     if(infer[1]) { # add CIs
         acv = .adj.critval(level, result$df, adjust, fam.info, side, corrmat, by.rows, sch.rank)
         ###adjust = acv$adjust # in older versions, I forced same adj method for tests
@@ -512,8 +550,13 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
                 .fmt.sigma(sigma)))
             estName = names(result)[1] = "prediction"
         }
-        result[[cnm[1]]] = result[[1]] + cv[, 1]*result$SE
-        result[[cnm[2]]] = result[[1]] + cv[, 2]*result$SE
+        if (!two.trans) {
+            result[[cnm[1]]] = result[[1]] + cv[, 1]*result$SE
+            result[[cnm[2]]] = result[[1]] + cv[, 2]*result$SE
+        }
+        else {
+            result[cnm] = summ.unlink[cnm]
+        }
         mesg = c(mesg, paste("Confidence level used:", level), acv$mesg)
         if (inv) {
             clims = with(link, cbind(linkinv(result[[cnm[1]]]), linkinv(result[[cnm[2]]])))
@@ -522,37 +565,47 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
             idx = if (all(tmp >= 0)) 1:2 else 2:1
             result[[cnm[1]]] = clims[, idx[1]]
             result[[cnm[2]]] = clims[, idx[2]]
-            mesg = c(mesg, paste("Intervals are back-transformed from the", link$name, "scale"))
+            mesg = c(mesg, paste("Intervals are back-transformed from the", linkname, "scale"))
         }
     }
     if(infer[2]) { # add tests
-        if (!all(null == 0)) {
+        tnm = ifelse (zFlag, "z.ratio", "t.ratio")
+        tail = ifelse(side == 0, -sign(abs(delta)), side)
+        if(!two.trans) {
             result[["null"]] = null
             if (inv && !is.null(link))
                 result[["null"]] = link$linkinv(null)
-        }
-        tnm = ifelse (zFlag, "z.ratio", "t.ratio")
-        tail = ifelse(side == 0, -sign(abs(delta)), side)
-        if (side == 0) {
-            if (delta == 0) # two-sided sig test
-                t.ratio = result[[tnm]] = (result[[1]] - null) / result$SE
-            else
-                t.ratio = result[[tnm]] = (abs(result[[1]] - null) - delta) / result$SE
+            if (all(result$null == 0))
+                result[["null"]] = NULL
+            if (side == 0) {
+                if (delta == 0) # two-sided sig test
+                    t.ratio = result[[tnm]] = (result[[1]] - null) / result$SE
+                else
+                    t.ratio = result[[tnm]] = (abs(result[[1]] - null) - delta) / result$SE
+            }
+            else {
+                t.ratio = result[[tnm]] = (result[[1]] - null + side * delta) / result$SE            
+            }
+            apv = .adj.p.value(t.ratio, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
+            adjust = apv$adjust   # in case it was abbreviated
+            result$p.value = apv$pval
+            mesg = c(mesg, apv$mesg)
         }
         else {
-            t.ratio = result[[tnm]] = (result[[1]] - null + side * delta) / result$SE            
+            result$null = ifelse(is.null(summ.unlink$null), link$linkinv(0), link$linkinv(summ.unlink$null))
+            result[[tnm]] = summ.unlink[[tnm]]
+            result$p.value = summ.unlink$p.value
+            apv = .adj.p.value(0, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
+            # we ignore everything about apv except the message
+            mesg = c(mesg, apv$mesg)
         }
-        apv = .adj.p.value(t.ratio, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
-        adjust = apv$adjust   # in case it was abbreviated
-        result$p.value = apv$pval
-        mesg = c(mesg, apv$mesg)
         if (delta > 0)
             mesg = c(mesg, paste("Statistics are tests of", c("nonsuperiority","equivalence","noninferiority")[side+2],
                                  "with a threshold of", signif(delta, 5)))
         if(tail != 0) 
             mesg = c(mesg, paste("P values are ", ifelse(tail<0,"left-","right-"),"tailed", sep=""))
         if (inv) 
-            mesg = c(mesg, paste("Tests are performed on the", link$name, "scale"))
+            mesg = c(mesg, paste("Tests are performed on the", linkname, "scale"))
     }
     if (inv) {
         result[["SE"]] = with(link, abs(mu.eta(result[[1]]) * result[["SE"]]))
@@ -580,6 +633,7 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
     attr(summ, "delta") = delta
     attr(summ, "type") = type
     attr(summ, "mesg") = unique(mesg)
+    attr(summ, "linkname") = linkname
     class(summ) = c("summary_emm", "data.frame")
     summ
 }
@@ -651,13 +705,27 @@ predict.emmGrid <- function(object, type,
 #' @order 5
 #' @param x object of the given class
 #' @param row.names passed to \code{\link{as.data.frame}}
-#' @param optional passed to \code{\link{as.data.frame}}
+#' @param optional required argument, but ignored in \code{as.data.frame.emmGrid}
+#' @param check.names passed to \code{\link{data.frame}}
 #' @return The \code{as.data.frame} method returns a plain data frame,
-#'   equivalent to \code{as.data.frame(summary(.))}.
+#'   equivalent to \code{as.data.frame(summary(.))}. 
+#' @note The \code{as.data.frame} method is intended primarily to allow for
+#'   \code{emmGrid} objects to be coerced to a data frame as needed internally.
+#'   However, we recommend \emph{against} users 
+#'   routinely using \code{as.data.frame}; instead, use \code{summary},
+#'   \code{confint}, or \code{test}, which already return a special
+#'   \code{data.frame} with added annotations. Those annotations display
+#'   important information such as adjustment methods and confidence levels.
+#'   If you need to see more digits, use \code{print(summary(object), digits = ...)};
+#'   and if you \emph{always} want to see more digits, use 
+#'   \code{emm_options(opt.digits = FALSE)}.
 #' @method as.data.frame emmGrid
 #' @export
-as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
-    as.data.frame(summary(x, ...), row.names = row.names, optional = optional)
+#' @examples
+#' # Show estimates to more digits
+#' print(test(con), digits = 7)
+as.data.frame.emmGrid = function(x, row.names = NULL, optional, check.names = TRUE, ...) {
+    as.data.frame(summary(x, ...), row.names = row.names, check.names = check.names)
 }
 
 
@@ -805,12 +873,14 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     n.contr = fam.info[2]
     et = as.numeric(fam.info[3])
 
-    if (n.contr == 1) # Force no adjustment when just one test
+    # Force no adjustment when just one test, unless we're using scheffe
+    if ((n.contr == 1) && !(adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) 
         adjust = "none"
     
-    # do a pmatch of the adjust method, case insensitive
+    # do a pmatch of the adjust method
     adj.meths = c("sidak", "tukey", "scheffe", "dunnettx", "mvt", p.adjust.methods)
-    k = pmatch(tolower(adjust), adj.meths)
+    
+    k = pmatch(adjust, adj.meths)
     if(is.na(k))
         stop("Adjust method '", adjust, "' is not recognized or not valid")
     adjust = adj.meths[k]
@@ -875,7 +945,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
         fs = fam.size
         scheffe.dim = sch.rank[1]
     }
-    do.msg = (chk.adj > 1) && (nc > 1) && !((fs < 3) && (chk.adj < 10)) 
+    do.msg = (chk.adj > 1) && !((fs < 3) && (chk.adj < 10)) ### WAS (chk.adj > 1) && (nc > 1) && !((fs < 3) && (chk.adj < 10)) 
     if (do.msg) {
 #         xtra = if(chk.adj < 10) paste("a family of", fam.size, "tests")
 #         else             paste(n.contr, "tests")
@@ -903,11 +973,11 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     et = as.numeric(fam.info[3])
     
     ragged.by = (is.character(fam.size))   # flag that we need to do groups separately
-    if (!ragged.by && n.contr == 1) # Force no adjustment when just one interval
+    if (!ragged.by && (n.contr == 1) && !(adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) # Force no adjustment when just one interval, unless using Scheffe
         adjust = "none"
     
     adj.meths = c("sidak", "tukey", "scheffe", "dunnettx", "mvt", "bonferroni", "none")
-    k = pmatch(tolower(adjust), adj.meths)
+    k = pmatch(adjust, adj.meths)
     if(is.na(k))
         k = which(adj.meths == "bonferroni") 
     adjust = adj.meths[k]
@@ -935,7 +1005,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
         fs = fam.size
         scheffe.dim = sch.rank[1]
     }
-    do.msg = (chk.adj > 1) && (nc > 1) && 
+    do.msg = (chk.adj > 1) && ###   (nc > 1) && 
         !((fs < 3) && (chk.adj < 10)) 
     
     if (do.msg) {
@@ -1118,24 +1188,26 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
 # Format a data.frame produced by summary.emmGrid
 #' @method print summary_emm
 #' @export
-print.summary_emm = function(x, ..., digits=NULL, quote=FALSE, right=TRUE) {
+print.summary_emm = function(x, ..., digits=NULL, quote=FALSE, right=TRUE, export = FALSE) {
+    test.stat.names = c("t.ratio", "z.ratio", "F.ratio", "T.square")  # format these w 3 dec places
     x.save = x
+    if(export) x.save = list()
     for(i in which(sapply(x, is.matrix))) 
         x[[i]] = NULL   # hide matrices
     for (i in seq_along(names(x)))   # zapsmall the numeric columns
         if (is.numeric(x[[i]]))  
             x[[i]] = zapsmall(x[[i]])
+    just = sapply(x, function(col) if(is.numeric(col)) "R" else "L")  ### was later
     if (!is.null(x$df)) x$df = round(x$df, 2)
-    if (!is.null(x$t.ratio)) 
-        x$t.ratio = format(round(x$t.ratio, 3), nsmall = 3, sci = FALSE)
-    if (!is.null(x$z.ratio)) 
-        x$z.ratio = format(round(x$z.ratio, 3), nsmall = 3, sci = FALSE)
+    for (nm in test.stat.names)
+        if(!is.null(x[[nm]]))
+            x[[nm]] = format(round(x[[nm]], 3), nsmall = 3, sci = FALSE)
     if (!is.null(x$p.value)) {
         fp = x$p.value = format(round(x$p.value, 4), nsmall = 4, sci = FALSE)
         x$p.value[fp=="0.0000"] = "<.0001"
     }
     estn = attr(x, "estName")
-    just = sapply(x, function(col) if(is.numeric(col)) "R" else "L")
+### moved up    just = sapply(x, function(col) if(is.numeric(col)) "R" else "L")
     est = x[[estn]]
     if (get_emm_option("opt.digits") && is.null(digits)) {
         if (!is.null(x[["SE"]]))
@@ -1160,30 +1232,62 @@ print.summary_emm = function(x, ..., digits=NULL, quote=FALSE, right=TRUE) {
     by.vars = attr(x, "by.vars")
     if (is.null(by.vars)) {
         m = .just.labs(m, just)
-        print(m, quote=FALSE, right=TRUE)
-        cat("\n")
+        if (export)
+            x.save$summary = m
+        else {
+            print(m, quote=FALSE, right=TRUE)
+            cat("\n")
+        }
     }
     else { # separate listing for each by variable
         m = .just.labs(m[, setdiff(names(x), by.vars), drop = FALSE], just)
+        if(export) 
+            x.save$summary = list()
         pargs = unname(as.list(x[,by.vars, drop=FALSE]))
         pargs$sep = ", "
         lbls = do.call(paste, pargs)
         for (lb in unique(lbls)) {
             rows = which(lbls==lb)
             levs = paste(by.vars, "=", xc[rows[1], by.vars])
-            cat(paste(paste(levs, collapse=", ")), ":\n", sep="")
-            print(m[rows, , drop=FALSE], ..., quote=quote, right=right)
-            cat("\n")
+            levs = paste(levs, collapse=", ")
+            if(export)
+                x.save$summary[[levs]] = m[rows, , drop = FALSE]
+            else {
+                cat(paste(levs, ":\n", sep=""))
+                print(m[rows, , drop=FALSE], ..., quote=quote, right=right)
+                cat("\n")
+            }
         }
     }
     
     msg = unique(attr(x, "mesg"))
-    if (!is.null(msg))
+    if (!is.null(msg) && !export)
         for (j in seq_len(length(msg))) cat(paste(msg[j], "\n"))
+    else (x.save$annotations = msg)
     
     invisible(x.save)
 }
 
+#' @method update summary_emm
+#' @export
+#' @rdname update.emmGrid
+#' @order 9
+#' @param by.vars,mesg Attributes that can be altered in \code{update.summary_emm}
+#' @section Method for \code{summary_emm} objects:
+#' This method exists so that we can change the way a summary is displayed,
+#' by changing the by variables or the annotations.
+#' 
+#' @examples
+#' ### Compactify results with a by variable
+#' update(joint_tests(pigs.rg, by = "source"), by = NULL)
+update.summary_emm = function(object, by.vars, mesg, ...) {
+    args = match.call()[-1]
+    args$object = NULL
+    for (nm in names(args))
+        attr(object, nm) = args[[nm]]
+    object
+}
+
 # determine optimum digits to display based on a conf or cred interval
 # (but always at least 3)
 .opt.dig = function(x) {
